/*
 * @plotXYAnimationPanel.java Version 1.1 08/26/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import plot.plotXYListStruct;
import plot.plotXYStruct;

import iqstrat.iqstratHeadersListStruct;

import quake.quakeSensorsStruct;         // Sensor Data Structure

import util.utilLandGridListStruct;      // TRS Land Grid List Structure

/** Class plotXYAnimationPanel
 *  <p> This Class will create the Stiff Plot.
 *
 *  @version 1.1 08/26/2015
 *  @author  John R. Victorine
 */

public class plotXYAnimationPanel extends JPanel
{
  public static final int _OFF   = 0;
  public static final int _ON    = 1;

  public static final int PREVIOUS = quake.gui.quakeAnimationFrame.PREVIOUS;
  public static final int NEXT     = quake.gui.quakeAnimationFrame.NEXT;

  // Input Variables

  private int                      iAction  = _ON;
  private int                      iGrid    = _ON;
  private plotXYListStruct         stList   = null;
  private iqstratHeadersListStruct stWells  = null;
  private quakeSensorsStruct       stSensor = null; // CO2 Sensors Location Structure
  private utilLandGridListStruct   stGrid   = null; // TRS Land Grid List Structure

  // Global Variables

  private int   iCurrent  = 0;    // Current image plotted

  private int   iPicture  = 0;    // Total Number of Sample Plot Images loaded
  private Image picture   = null; // Array of Sample Plot Images

//  private int   delay     = 1000; // milliseconds (1000 = 1 Second)
  private int   delay     = 500; // milliseconds (1000 = 1 Second)
  private Timer timer     = null; // Timer for Animation

  /** Constructor plotXYAnimationPanel()
   *  <p> This is the Constructor for this class.
   *  @param st       = Brine Data Structure
   *  @param stWells  = SWD Wells & Wellington KGS Wells
   *  @param stSensor = CO2 Sensor Location Structure
   *  @param stGrid   = TRS Land Grid List Structure
   */

  public plotXYAnimationPanel( int                      iAction,
                               int                      iGrid,
                               plotXYListStruct         stList,
                               iqstratHeadersListStruct stWells,
                               quakeSensorsStruct       stSensor,
                               utilLandGridListStruct   stGrid )
  {
    try
    {
      this.iAction  = iAction;
      this.iGrid    = iGrid;
      this.stList   = stList;
      this.stWells  = stWells;
      this.stSensor = stSensor;
      this.stGrid   = stGrid;

      init();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method init()
   *  <p> This method will initialize and create the images for animation
   */

  public void init()
  {
	// Create the Images for animation

    if (stList != null)
    {
      if (stList.iCount > 0)
      {
        for (int i=0; i<stList.iCount; i++)
        {
	      iPicture++;
	    }
      }

      // Set up a timer that calls this object's action handler.

      initTimer();
    }
  }

  /** Method paint()
   *  Class Paint Method
   *  @param g = graphics object
   */

  public void paint(Graphics g)
  {
    super.paintComponent(g);
    Graphics2D g2 = (Graphics2D) g;

    picture = plot.plotXYDataImage.makePlotImage(
		       iAction, iGrid, stList.stItem[iCurrent], stWells, stSensor, stGrid );

    if (picture != null)
    {
      g2.drawImage(picture, 0, 0, this);
    }
    else
      System.out.println("Picture: "+iCurrent+" Does NOT Exist");
  }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = 900;
    int h = 700;

    picture = plot.plotXYDataImage.makePlotImage(
		       iAction, iGrid, stList.stItem[iCurrent], stWells, stSensor, stGrid );

    if (picture != null)
    {
      image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

      Graphics2D g2 = image.createGraphics();
      g2.setColor( Color.white );
      g2.fillRect( 0, 0, w, h );

      g2.drawImage(picture, 0, 0, this);

      g2.dispose();
    }

    return (image);
  }

  /* ================================================================== *
   * ------------------------ CLOSE METHODS --------------------------- *
   * ================================================================== */

  /** Method close()
   *  <p> deallocate memory and close out frame
   */

  public void close()
  {
    // Input Variables

    stList   = null; // Earthquake Plot Data List Structure
    stWells  = null; // Well Header Data List Structure
    stSensor = null; // CO2 Sensors location Structure
    stGrid   = null; // TRS Land Grid List Structure

    // Global Variables

    iCurrent = 0;    // Current image plotted
    picture  = null;
    iPicture = 0;    // Total Number of Sample Plot Images loaded
    timer    = null; // Timer for Animation
  }

  /* ================================================================== *
   * ------------------------- SET METHODS ---------------------------- *
   * ================================================================== */

  /** Method setChanges()
   *  <p> This method will set the changes in the limits
   */

  public void setChanges(int iAction, int iGrid, plotXYListStruct stList)
  {
    this.iAction = iAction;
    this.iGrid   = iGrid;
    this.stList  = stList;
  }

  /** Method setEvent()
   *  <p> This method will set the event to be displayed
   *  @param iEvent = event number in the plot list
   */

  public void setEvent( int iEvent )
  {
	if (iEvent < iPicture-1)
	  setCurrent(iEvent);
  }

  /** Method setDelay()
   *  <p> This method will set the fps of the animation
   *  @param delay = the number of seconds delay
   */

  public void setDelay(int delay)
  {
	this.delay = delay;
    timer.setDelay(delay);
  }

  /* ===================================================================== *
   * ----------------------- ANIMATION ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method initTimer()
   *  <p> this method will initialize the timer
   */

  private void initTimer()
  {
    timer = new Timer(delay, new animateAction());
    timer.setInitialDelay(0);
    timer.setCoalesce(true);
  }

  /** Method startAnimation() throws Exception
   *  <p> Starts the timer that generates action events
   * @throws Exception - exception starting the timer
   */

  public void startAnimation() throws Exception
  {
    if (!timer.isRunning())
    {
      timer.start();
    }
  }

  /** Method pauseAnimation()
   *  <p> Stops the timer that generate action events
   */

  public void pauseAnimation() throws Exception
  {
    if (timer.isRunning())
    {
      timer.stop();
    }
  }

  /** Method stopAnimation() throws Exception
   *  <p> Stops the timer that generate action events
   *      and resets the current image number to 0
   * @throws Exception - exception stopping the timer
   */

  public void stopAnimation() throws Exception
  {
    if (timer.isRunning())
    {
      timer.stop();
      setCurrent(0);
    }
  }

  /** Method stepAction()
   *  <p> This method will issue a step action to the plot
   */

  public void stepAction(int iStep)
  {
	switch (iStep)
	{
	  case PREVIOUS:
        if (iCurrent == 0)
          setCurrent(iPicture-1);
        else
          setCurrent(iCurrent-1);
	    break;

	  case NEXT:
        if (iCurrent < iPicture-1)
          setCurrent(iCurrent+1);
        else
          setCurrent(0);
	    break;
	}
  }

  /** Class animateAction()
   *  <p> This method will perform the changes in the displayed image
   */

  class animateAction implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      if (iCurrent < iPicture-1)
        setCurrent(iCurrent+1);
      else
        setCurrent(0);
    }
  }

  /** Method setCurrent(int iValue)
   *  <p> Sets the current image number.
   *
   *  @param iValue - image to be
   */

  private void setCurrent(int iValue)
  {
    this.iCurrent = iValue;
    repaint();
  }
}

/*
 *  @version 1.1 08/26/2015
 *  @author  John Victorine
 */
