/*
 * @plotXYZAnimationFrame.java Version 1.1 08/26/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;

import plot.plotXYListStruct;
import plot.plotXYStruct;
import plot.plotXYZAnimationPanel;

import iqstrat.iqstratHeadersListStruct;

import quake.quakeSensorsStruct;         // Sensor Data Structure

import util.utilLandGridListStruct;      // TRS Land Grid List Structure

/** Class plotXYZAnimationFrame
 *  <p> This class will display the Cross Plot Frame
 *
 *  @version 1.1 08/26/2015
 *  @author  John R. Victorine
 */

public class plotXYZAnimationFrame extends JFrame //implements ActionListener
{
  private int                      iAction  = 0;
  private int                      iGrid    = 0;
  private plotXYListStruct         stList   = null;
  private plotXYZAnimationPanel    panel    = null;
  private iqstratHeadersListStruct stWells  = null;
  private quakeSensorsStruct       stSensor = null; // CO2 Sensors Location Structure
  private utilLandGridListStruct   stGrid   = null; // TRS Land Grid List Structure

  /** Constructor plotXYZAnimationFrame()
   *  <p> This is the constructor for this class
   * @param notifier = Oberservable
   * @param iPlot    = Plot Type
   * @param stList   = XY Plot Data Structure
   * @param stGrid  = TRS Land Grid List Structure
   */

  public plotXYZAnimationFrame( int                      iAction,
                                int                      iGrid,
	                            plotXYListStruct         stList,
                                iqstratHeadersListStruct stWells,
                                quakeSensorsStruct       stSensor,
                                utilLandGridListStruct   stGrid )
  {
    try
    {
      this.iAction  = iAction;
      this.iGrid    = iGrid;
      this.stList   = stList;
      this.stWells  = stWells;
      this.stSensor = stSensor;
      this.stGrid   = stGrid;

      jbInit();

      addWindowListener( new plotXYZAnimationFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    JScrollPane jScrollPane = new JScrollPane();

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle("KS Earthquake 3D Plot Animation");

    panel = new plotXYZAnimationPanel( iAction, iGrid, stList, stWells, stSensor, stGrid );

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel, null);

    // Add a Window Listener to close dialog

    this.addWindowListener( new WindowAdapter()
    {
      public void windowClosing(WindowEvent e) { close(); }
    });

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(900, 700));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) - 10,
                     (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD plotXYZAnimationFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class plotXYZAnimationFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      // NO ACTION
    }
  }

  /** Method close()
   *  <p> This method will force deallocation of memory in Java
   */

  public void close()
  {
    stList   = null;
    stWells  = null;
    stSensor = null; // CO2 Sensors location Structure
    stGrid   = null; // TRS Land Grid List Structure

    if (panel != null)
      panel.close();
    panel = null;

    dispose();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /* ================================================================== *
   * ------------------------- SET METHODS ---------------------------- *
   * ================================================================== */

  /** Method setChanges()
   *  <p> This method will set the changes in the limits
   */

  public void setChanges(int iAction, int iGrid, plotXYListStruct stList)
  {
	if (panel != null)
	{
	  panel.setChanges(iAction, iGrid, stList);
	}
  }

  /** Method setEvent()
   *  <p> This method will set the event to be displayed
   *  @param iEvent = event number in the plot list
   */

  public void setEvent( int iEvent )
  {
	if (panel != null)
	{
	  panel.setEvent( iEvent );
	}
  }

  /** Method setDelay()
   *  <p> This method will set the fps of the animation
   *  @param delay = the number of seconds delay
   */

  public void setDelay(int delay)
  {
	if (panel != null)
	{
	  panel.setDelay( delay );
	}
  }

  /* ================================================================== *
   * ------------------- ANIMATION ACTION METHODS --------------------- *
   * ================================================================== */

  /** Method startAnimation() throws Exception
   *  <p> Starts the timer that generates action events
   * @throws Exception - exception starting the timer
   */

  public void startAnimation()
  {
	if (panel != null)
	{
      try
      {
  	    panel.startAnimation();
	  }
	  catch(Exception e) {}
	}
  }

  /** Method pauseAnimation()
   *  <p> Stops the timer that generate action events
   */

  public void pauseAnimation()
  {
	if (panel != null)
	{
      try
      {
  	    panel.pauseAnimation();
	  }
	  catch(Exception e) {}
	}
  }

  /** Method stopAnimation() throws Exception
   *  <p> Stops the timer that generate action events
   *      and resets the current image number to 0
   * @throws Exception - exception stopping the timer
   */

  public void stopAnimation()
  {
	if (panel != null)
	{
      try
      {
  	    panel.stopAnimation();
	  }
	  catch(Exception e) {}
	}
  }

  /** Method stepAction()
   *  <p> This method will issue a step action to the plot
   */

  public void stepAction(int iStep) { panel.stepAction(iStep); }
}

/*
 *  @version 1.1 08/26/2015
 *  @author  John Victorine
 */
